@@ -32,7 +32,7 @@ G_BEGIN_DECLS
 #define BG_TYPE_PICTURES_SOURCE (bg_pictures_source_get_type ())
 G_DECLARE_FINAL_TYPE (BgPicturesSource, bg_pictures_source, BG, PICTURES_SOURCE, BgSource)
 
-BgPicturesSource *bg_pictures_source_new            (GtkWindow *window);
+BgPicturesSource *bg_pictures_source_new            (GtkWidget *widget);
 char             *bg_pictures_source_get_cache_path (void);
 char             *bg_pictures_source_get_unique_path(const char *uri);
 gboolean          bg_pictures_source_add            (BgPicturesSource     *bg_source,
