@@ -28,8 +28,9 @@
 
 typedef struct
 {
-  GtkListStore *store;
-  GtkWidget *window;
+  GnomeDesktopThumbnailFactory *thumbnail_factory;
+  GListStore *store;
+  GtkWidget *widget;
   gint thumbnail_height;
   gint thumbnail_width;
 } BgSourcePrivate;
@@ -39,7 +40,7 @@ G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (BgSource, bg_source, G_TYPE_OBJECT)
 enum
 {
   PROP_LISTSTORE = 1,
-  PROP_WINDOW
+  PROP_WIDGET
 };
 
 
@@ -52,10 +53,10 @@ bg_source_calculate_thumbnail_dimensions (BgSource *source)
   priv->thumbnail_height = THUMBNAIL_HEIGHT;
   priv->thumbnail_width = THUMBNAIL_WIDTH;
 
-  if (priv->window == NULL)
+  if (priv->widget == NULL)
     return;
 
-  scale_factor = gtk_widget_get_scale_factor (priv->window);
+  scale_factor = gtk_widget_get_scale_factor (priv->widget);
   if (scale_factor > 1)
     {
       priv->thumbnail_height *= scale_factor;
@@ -101,8 +102,8 @@ bg_source_set_property (GObject      *object,
 
   switch (property_id)
     {
-    case PROP_WINDOW:
-      priv->window = GTK_WIDGET (g_value_get_object (value));
+    case PROP_WIDGET:
+      priv->widget = GTK_WIDGET (g_value_get_object (value));
       break;
 
     default:
@@ -116,6 +117,7 @@ bg_source_dispose (GObject *object)
   BgSource *source = BG_SOURCE (object);
   BgSourcePrivate *priv = bg_source_get_instance_private (source);
 
+  g_clear_object (&priv->thumbnail_factory);
   g_clear_object (&priv->store);
 
   G_OBJECT_CLASS (bg_source_parent_class)->dispose (object);
@@ -135,26 +137,27 @@ bg_source_class_init (BgSourceClass *klass)
   pspec = g_param_spec_object ("liststore",
                                "Liststore",
                                "Liststore used in the source",
-                               GTK_TYPE_LIST_STORE,
+                               G_TYPE_LIST_STORE,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_LISTSTORE, pspec);
 
-  pspec = g_param_spec_object ("window",
-                               "Window",
-                               "Toplevel window used to view the source",
-                               GTK_TYPE_WINDOW,
+  pspec = g_param_spec_object ("widget",
+                               "Widget",
+                               "Widget used to view the source",
+                               GTK_TYPE_WIDGET,
                                G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (object_class, PROP_WINDOW, pspec);
+  g_object_class_install_property (object_class, PROP_WIDGET, pspec);
 }
 
 static void
 bg_source_init (BgSource *self)
 {
   BgSourcePrivate *priv = bg_source_get_instance_private (self);
-  priv->store = gtk_list_store_new (3, CAIRO_GOBJECT_TYPE_SURFACE, G_TYPE_OBJECT, G_TYPE_STRING);
+  priv->store = g_list_store_new (CC_TYPE_BACKGROUND_ITEM);
+  priv->thumbnail_factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
 }
 
-GtkListStore*
+GListStore*
 bg_source_get_liststore (BgSource *source)
 {
   BgSourcePrivate *priv;
@@ -173,7 +176,7 @@ bg_source_get_scale_factor (BgSource *source)
   g_return_val_if_fail (BG_IS_SOURCE (source), 1);
 
   priv = bg_source_get_instance_private (source);
-  return gtk_widget_get_scale_factor (priv->window);
+  return gtk_widget_get_scale_factor (priv->widget);
 }
 
 gint
@@ -197,3 +200,14 @@ bg_source_get_thumbnail_width (BgSource *source)
   priv = bg_source_get_instance_private (source);
   return priv->thumbnail_width;
 }
+
+GnomeDesktopThumbnailFactory*
+bg_source_get_thumbnail_factory (BgSource *source)
+{
+  BgSourcePrivate *priv;
+
+  g_return_val_if_fail (BG_IS_SOURCE (source), NULL);
+
+  priv = bg_source_get_instance_private (source);
+  return priv->thumbnail_factory;
+}
