@@ -1990,7 +1990,7 @@ cc_color_panel_class_init (CcColorPanelClass *klass)
   object_class->dispose = cc_color_panel_dispose;
   object_class->finalize = cc_color_panel_finalize;
 
-  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/color/color.ui");
+  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/color/cc-color-panel.ui");
 
   gtk_widget_class_bind_template_child (widget_class, CcColorPanel, assistant_calib);
   gtk_widget_class_bind_template_child (widget_class, CcColorPanel, box_calib_brightness);
