@@ -83,7 +83,7 @@ cc_mouse_panel_constructed (GObject *object)
   gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
   gtk_widget_set_visible (button, TRUE);
 
-  cc_shell_embed_widget_in_header (shell, button);
+  cc_shell_embed_widget_in_header (shell, button, GTK_POS_RIGHT);
 
   g_signal_connect (GTK_BUTTON (button), "toggled",
                     G_CALLBACK (shell_test_button_toggled),
