@@ -26,21 +26,13 @@
 #define IPP_STATE_IDLE IPP_IDLE
 #endif
 
-typedef struct _PpPrinter        PpPrinter;
-typedef struct _PpPrinterPrivate PpPrinterPrivate;
-
 struct _PpPrinter
 {
-  GObject           parent_instance;
-  PpPrinterPrivate *priv;
+  GObject  parent_instance;
+  gchar   *printer_name;
 };
 
-struct _PpPrinterPrivate
-{
-  gchar *printer_name;
-};
-
-G_DEFINE_TYPE_WITH_PRIVATE (PpPrinter, pp_printer, G_TYPE_OBJECT)
+G_DEFINE_TYPE (PpPrinter, pp_printer, G_TYPE_OBJECT)
 
 enum
 {
@@ -51,9 +43,9 @@ enum
 static void
 pp_printer_dispose (GObject *object)
 {
-  PpPrinterPrivate *priv = PP_PRINTER (object)->priv;
+  PpPrinter *self = PP_PRINTER (object);
 
-  g_free (priv->printer_name);
+  g_clear_pointer (&self->printer_name, g_free);
 
   G_OBJECT_CLASS (pp_printer_parent_class)->dispose (object);
 }
@@ -75,7 +67,7 @@ pp_printer_get_property (GObject    *object,
   switch (property_id)
     {
       case PROP_NAME:
-        g_value_set_string (value, self->priv->printer_name);
+        g_value_set_string (value, self->printer_name);
         break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -94,8 +86,8 @@ pp_printer_set_property (GObject      *object,
   switch (property_id)
     {
       case PROP_NAME:
-        g_free (self->priv->printer_name);
-        self->priv->printer_name = g_value_dup_string (value);
+        g_free (self->printer_name);
+        self->printer_name = g_value_dup_string (value);
         break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -123,11 +115,8 @@ pp_printer_class_init (PpPrinterClass *klass)
 }
 
 static void
-pp_printer_init (PpPrinter *printer)
+pp_printer_init (PpPrinter *self)
 {
-  printer->priv = G_TYPE_INSTANCE_GET_PRIVATE (printer,
-                                               PP_TYPE_PRINTER,
-                                               PpPrinterPrivate);
 }
 
 PpPrinter *
@@ -144,22 +133,20 @@ printer_rename_thread (GTask        *task,
                        gpointer      task_data,
                        GCancellable *cancellable)
 {
-  PpPrinter *printer = PP_PRINTER (source_object);
-  gboolean   result;
-  gchar     *new_printer_name = task_data;
-  gchar     *old_printer_name;
+  PpPrinter        *self = PP_PRINTER (source_object);
+  gboolean          result;
+  gchar            *new_printer_name = task_data;
+  g_autofree gchar *old_printer_name = NULL;
 
-  g_object_get (printer, "printer-name", &old_printer_name, NULL);
+  g_object_get (self, "printer-name", &old_printer_name, NULL);
 
   result = printer_rename (old_printer_name, new_printer_name);
 
   if (result)
     {
-      g_object_set (printer, "printer-name", new_printer_name, NULL);
+      g_object_set (self, "printer-name", new_printer_name, NULL);
     }
 
-  g_free (old_printer_name);
-
   g_task_return_boolean (task, result);
 }
 
@@ -168,12 +155,11 @@ printer_rename_dbus_cb (GObject      *source_object,
                         GAsyncResult *res,
                         gpointer      user_data)
 {
-  PpPrinter        *printer;
+  PpPrinter        *self;
   GVariant         *output;
   gboolean          result = FALSE;
   g_autoptr(GError) error = NULL;
   GTask            *task = user_data;
-  gchar            *old_printer_name;
 
   output = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source_object),
                                           res,
@@ -182,10 +168,11 @@ printer_rename_dbus_cb (GObject      *source_object,
 
   if (output != NULL)
     {
+      g_autofree gchar *old_printer_name = NULL;
       const gchar *ret_error;
 
-      printer = g_task_get_source_object (task);
-      g_object_get (printer, "printer-name", &old_printer_name, NULL);
+      self = g_task_get_source_object (task);
+      g_object_get (self, "printer-name", &old_printer_name, NULL);
 
       g_variant_get (output, "(&s)", &ret_error);
       if (ret_error[0] != '\0')
@@ -195,12 +182,11 @@ printer_rename_dbus_cb (GObject      *source_object,
       else
         {
           result = TRUE;
-          g_object_set (printer, "printer-name", g_task_get_task_data (task), NULL);
+          g_object_set (self, "printer-name", g_task_get_task_data (task), NULL);
         }
 
       g_task_return_boolean (task, result);
 
-      g_free (old_printer_name);
       g_variant_unref (output);
     }
   else
@@ -228,11 +214,12 @@ get_bus_cb (GObject      *source_object,
   GDBusConnection  *bus;
   g_autoptr(GError) error = NULL;
   GTask            *task = user_data;
-  gchar            *printer_name;
 
   bus = g_bus_get_finish (res, &error);
   if (bus != NULL)
     {
+      g_autofree gchar *printer_name = NULL;
+
       g_object_get (g_task_get_source_object (task),
                     "printer-name", &printer_name,
                     NULL);
@@ -250,8 +237,6 @@ get_bus_cb (GObject      *source_object,
                               g_task_get_cancellable (task),
                               printer_rename_dbus_cb,
                               task);
-
-      g_free (printer_name);
     }
   else
     {
@@ -261,7 +246,7 @@ get_bus_cb (GObject      *source_object,
 }
 
 void
-pp_printer_rename_async (PpPrinter           *printer,
+pp_printer_rename_async (PpPrinter           *self,
                          const gchar         *new_printer_name,
                          GCancellable        *cancellable,
                          GAsyncReadyCallback  callback,
@@ -271,7 +256,7 @@ pp_printer_rename_async (PpPrinter           *printer,
 
   g_return_if_fail (new_printer_name != NULL);
 
-  task = g_task_new (G_OBJECT (printer), cancellable, callback, user_data);
+  task = g_task_new (G_OBJECT (self), cancellable, callback, user_data);
   g_task_set_task_data (task, g_strdup (new_printer_name), g_free);
 
   g_bus_get (G_BUS_TYPE_SYSTEM,
@@ -281,11 +266,11 @@ pp_printer_rename_async (PpPrinter           *printer,
 }
 
 gboolean
-pp_printer_rename_finish (PpPrinter     *printer,
+pp_printer_rename_finish (PpPrinter     *self,
                           GAsyncResult  *res,
                           GError       **error)
 {
-  g_return_val_if_fail (g_task_is_valid (res, printer), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
   g_object_unref (res);
 
   return g_task_propagate_boolean (G_TASK (res), error);
@@ -303,26 +288,24 @@ get_jobs_thread (GTask        *task,
                  gpointer      task_data,
                  GCancellable *cancellable)
 {
-  ipp_attribute_t *attr = NULL;
-  static gchar    *printer_attributes[] = { "auth-info-required" };
-  GetJobsData     *get_jobs_data = task_data;
-  cups_job_t      *jobs = NULL;
-  PpPrinter       *printer = PP_PRINTER (source_object);
-  gboolean         auth_info_is_required;
-  PpJob           *job;
-  ipp_t           *job_request;
-  ipp_t           *job_response;
-  ipp_t           *printer_request;
-  ipp_t           *printer_response;
-  gchar           *job_uri;
-  gchar           *printer_uri;
-  gchar          **auth_info_required = NULL;
-  gchar           *printer_name;
-  GList           *list = NULL;
-  gint             num_jobs;
-  gint             i, j;
+  ipp_attribute_t  *attr = NULL;
+  static gchar     *printer_attributes[] = { "auth-info-required" };
+  GetJobsData      *get_jobs_data = task_data;
+  cups_job_t       *jobs = NULL;
+  PpPrinter        *self = PP_PRINTER (source_object);
+  gboolean          auth_info_is_required;
+  PpJob            *job;
+  ipp_t            *job_request;
+  ipp_t            *job_response;
+  ipp_t            *printer_request;
+  ipp_t            *printer_response;
+  gchar           **auth_info_required = NULL;
+  g_autofree gchar *printer_name = NULL;
+  GList            *list = NULL;
+  gint              num_jobs;
+  gint              i, j;
 
-  g_object_get (printer, "printer-name", &printer_name, NULL);
+  g_object_get (self, "printer-name", &printer_name, NULL);
 
   num_jobs = cupsGetJobs (&jobs,
                           printer_name,
@@ -334,7 +317,7 @@ get_jobs_thread (GTask        *task,
       auth_info_is_required = FALSE;
       if (jobs[i].state == IPP_JOB_HELD)
         {
-          job_uri = g_strdup_printf ("ipp://localhost/jobs/%d", jobs[i].id);
+          g_autofree gchar *job_uri = g_strdup_printf ("ipp://localhost/jobs/%d", jobs[i].id);
 
           job_request = ippNewRequest (IPP_GET_JOB_ATTRIBUTES);
           ippAddString (job_request, IPP_TAG_OPERATION, IPP_TAG_URI,
@@ -345,8 +328,6 @@ get_jobs_thread (GTask        *task,
                         "requested-attributes", NULL, "job-hold-until");
           job_response = cupsDoRequest (CUPS_HTTP_DEFAULT, job_request, "/");
 
-          g_free (job_uri);
-
           if (job_response != NULL)
             {
               attr = ippFindAttribute (job_response, "job-hold-until", IPP_TAG_ZERO);
@@ -356,7 +337,7 @@ get_jobs_thread (GTask        *task,
 
                   if (auth_info_required == NULL)
                     {
-                      printer_uri = g_strdup_printf ("ipp://localhost/printers/%s", printer_name);
+                      g_autofree gchar *printer_uri = g_strdup_printf ("ipp://localhost/printers/%s", printer_name);
 
                       printer_request = ippNewRequest (IPP_GET_PRINTER_ATTRIBUTES);
                       ippAddString (printer_request, IPP_TAG_OPERATION, IPP_TAG_URI,
@@ -367,8 +348,6 @@ get_jobs_thread (GTask        *task,
                                      "requested-attributes", 1, NULL, (const char **) printer_attributes);
                       printer_response = cupsDoRequest (CUPS_HTTP_DEFAULT, printer_request, "/");
 
-                      g_free (printer_uri);
-
                       if (printer_response != NULL)
                         {
                           attr = ippFindAttribute (printer_response, "auth-info-required", IPP_TAG_ZERO);
@@ -400,7 +379,6 @@ get_jobs_thread (GTask        *task,
 
   g_strfreev (auth_info_required);
   cupsFreeJobs (num_jobs, jobs);
-  g_free (printer_name);
 
   if (g_task_set_return_on_cancel (task, FALSE))
     {
@@ -409,7 +387,7 @@ get_jobs_thread (GTask        *task,
 }
 
 void
-pp_printer_get_jobs_async (PpPrinter           *printer,
+pp_printer_get_jobs_async (PpPrinter           *self,
                            gboolean             myjobs,
                            gint                 which_jobs,
                            GCancellable        *cancellable,
@@ -423,7 +401,7 @@ pp_printer_get_jobs_async (PpPrinter           *printer,
   get_jobs_data->myjobs = myjobs;
   get_jobs_data->which_jobs = which_jobs;
 
-  task = g_task_new (G_OBJECT (printer), cancellable, callback, user_data);
+  task = g_task_new (G_OBJECT (self), cancellable, callback, user_data);
   g_task_set_task_data (task, get_jobs_data, g_free);
   g_task_set_return_on_cancel (task, TRUE);
   g_task_run_in_thread (task, get_jobs_thread);
@@ -431,11 +409,11 @@ pp_printer_get_jobs_async (PpPrinter           *printer,
 }
 
 GList *
-pp_printer_get_jobs_finish (PpPrinter          *printer,
+pp_printer_get_jobs_finish (PpPrinter          *self,
                             GAsyncResult       *res,
                             GError            **error)
 {
-  g_return_val_if_fail (g_task_is_valid (res, printer), NULL);
+  g_return_val_if_fail (g_task_is_valid (res, self), NULL);
 
   return g_task_propagate_pointer (G_TASK (res), error);
 }
@@ -449,7 +427,6 @@ pp_printer_delete_dbus_cb (GObject      *source_object,
   gboolean          result = FALSE;
   g_autoptr(GError) error = NULL;
   GTask            *task = user_data;
-  gchar            *printer_name;
 
   output = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source_object),
                                           res,
@@ -458,7 +435,8 @@ pp_printer_delete_dbus_cb (GObject      *source_object,
 
   if (output != NULL)
     {
-      const gchar *ret_error;
+      g_autofree gchar *printer_name = NULL;
+      const gchar      *ret_error;
 
       g_object_get (g_task_get_source_object (task), "printer-name", &printer_name, NULL);
 
@@ -470,7 +448,6 @@ pp_printer_delete_dbus_cb (GObject      *source_object,
 
       g_task_return_boolean (task, result);
 
-      g_free (printer_name);
       g_variant_unref (output);
     }
   else
@@ -489,11 +466,12 @@ pp_printer_delete_cb (GObject      *source_object,
   GDBusConnection  *bus;
   g_autoptr(GError) error = NULL;
   GTask            *task = user_data;
-  gchar            *printer_name;
 
   bus = g_bus_get_finish (res, &error);
   if (bus != NULL)
     {
+      g_autofree gchar *printer_name = NULL;
+
       g_object_get (g_task_get_source_object (task),
                     "printer-name", &printer_name,
                     NULL);
@@ -510,8 +488,6 @@ pp_printer_delete_cb (GObject      *source_object,
                               g_task_get_cancellable (task),
                               pp_printer_delete_dbus_cb,
                               task);
-
-      g_free (printer_name);
     }
   else
     {
@@ -521,14 +497,14 @@ pp_printer_delete_cb (GObject      *source_object,
 }
 
 void
-pp_printer_delete_async (PpPrinter           *printer,
+pp_printer_delete_async (PpPrinter           *self,
                          GCancellable        *cancellable,
                          GAsyncReadyCallback  callback,
                          gpointer             user_data)
 {
   GTask *task;
 
-  task = g_task_new (G_OBJECT (printer), cancellable, callback, user_data);
+  task = g_task_new (G_OBJECT (self), cancellable, callback, user_data);
 
   g_bus_get (G_BUS_TYPE_SYSTEM,
              cancellable,
@@ -537,11 +513,11 @@ pp_printer_delete_async (PpPrinter           *printer,
 }
 
 gboolean
-pp_printer_delete_finish (PpPrinter     *printer,
+pp_printer_delete_finish (PpPrinter     *self,
                           GAsyncResult  *res,
                           GError       **error)
 {
-  g_return_val_if_fail (g_task_is_valid (res, printer), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
 
   return g_task_propagate_boolean (G_TASK (res), error);
 }
@@ -567,21 +543,19 @@ print_file_thread (GTask        *task,
                    gpointer      task_data,
                    GCancellable *cancellable)
 {
-  PrintFileData *print_file_data;
-  cups_ptype_t  type = 0;
-  cups_dest_t  *dest = NULL;
-  const gchar  *printer_type = NULL;
-  PpPrinter    *printer;
-  gboolean      ret = FALSE;
-  gchar        *printer_name = NULL;
-  gchar        *printer_uri = NULL;
-  gchar        *resource = NULL;
-  ipp_t        *response = NULL;
-  ipp_t        *request;
+  PpPrinter        *self = PP_PRINTER (source_object);
+  PrintFileData    *print_file_data;
+  cups_ptype_t      type = 0;
+  cups_dest_t      *dest = NULL;
+  const gchar      *printer_type = NULL;
+  gboolean          ret = FALSE;
+  g_autofree gchar *printer_name = NULL;
+  g_autofree gchar *printer_uri = NULL;
+  g_autofree gchar *resource = NULL;
+  ipp_t            *response = NULL;
+  ipp_t            *request;
 
-  printer = PP_PRINTER (source_object);
-
-  g_object_get (printer, "printer-name", &printer_name, NULL);
+  g_object_get (self, "printer-name", &printer_name, NULL);
   dest = cupsGetNamedDest (CUPS_HTTP_DEFAULT, printer_name, NULL);
   if (dest != NULL)
     {
@@ -626,10 +600,6 @@ print_file_thread (GTask        *task,
       ippDelete (response);
     }
 
-  g_free (printer_name);
-  g_free (printer_uri);
-  g_free (resource);
-
   if (g_task_set_return_on_cancel (task, FALSE))
     {
       g_task_return_boolean (task, ret);
@@ -637,7 +607,7 @@ print_file_thread (GTask        *task,
 }
 
 void
-pp_printer_print_file_async (PpPrinter           *printer,
+pp_printer_print_file_async (PpPrinter           *self,
                              const gchar         *filename,
                              const gchar         *job_name,
                              GCancellable        *cancellable,
@@ -651,7 +621,7 @@ pp_printer_print_file_async (PpPrinter           *printer,
   print_file_data->filename = g_strdup (filename);
   print_file_data->job_name = g_strdup (job_name);
 
-  task = g_task_new (G_OBJECT (printer), cancellable, callback, user_data);
+  task = g_task_new (G_OBJECT (self), cancellable, callback, user_data);
 
   g_task_set_return_on_cancel (task, TRUE);
   g_task_set_task_data (task, print_file_data, (GDestroyNotify) print_file_data_free);
@@ -661,11 +631,11 @@ pp_printer_print_file_async (PpPrinter           *printer,
 }
 
 gboolean
-pp_printer_print_file_finish (PpPrinter     *printer,
+pp_printer_print_file_finish (PpPrinter     *self,
                               GAsyncResult  *res,
                               GError       **error)
 {
-  g_return_val_if_fail (g_task_is_valid (res, printer), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
 
   return g_task_propagate_boolean (G_TASK (res), error);
 }
