@@ -209,6 +209,7 @@ cc_sharing_networks_new_row (const char        *uuid,
 
   row = gtk_list_box_row_new ();
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_widget_show (box);
   gtk_container_set_border_width (GTK_CONTAINER (box), 3);
   gtk_widget_set_margin_start (box, 6);
   gtk_container_add (GTK_CONTAINER (row), box);
@@ -222,16 +223,19 @@ cc_sharing_networks_new_row (const char        *uuid,
   }
 
   w = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+  gtk_widget_show (w);
   gtk_widget_set_margin_end (w, 12);
   gtk_container_add (GTK_CONTAINER (box), w);
 
   /* Label */
   w = gtk_label_new (network_name);
+  gtk_widget_show (w);
   gtk_widget_set_margin_end (w, 12);
   gtk_container_add (GTK_CONTAINER (box), w);
 
   /* Remove button */
   w = gtk_button_new_from_icon_name ("window-close-symbolic", GTK_ICON_SIZE_SMALL_TOOLBAR);
+  gtk_widget_show (w);
   gtk_button_set_relief (GTK_BUTTON (w), GTK_RELIEF_NONE);
   gtk_widget_set_margin_top (w, 3);
   gtk_widget_set_margin_bottom (w, 3);
@@ -244,8 +248,6 @@ cc_sharing_networks_new_row (const char        *uuid,
 
   g_object_set_data_full (G_OBJECT (row), "uuid", g_strdup (uuid), g_free);
 
-  gtk_widget_show_all (row);
-
   return row;
 }
 
@@ -256,23 +258,27 @@ cc_sharing_networks_new_current_row (CcSharingNetworks *self)
 
   row = gtk_list_box_row_new ();
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_widget_show (box);
   gtk_container_set_border_width (GTK_CONTAINER (box), 3);
   gtk_widget_set_margin_start (box, 6);
   gtk_container_add (GTK_CONTAINER (row), box);
 
   /* Icon */
   w = gtk_image_new_from_icon_name ("image-missing", GTK_ICON_SIZE_MENU);
+  gtk_widget_show (w);
   gtk_widget_set_margin_end (w, 12);
   gtk_container_add (GTK_CONTAINER (box), w);
   self->current_icon = w;
 
   /* Label */
   w = gtk_label_new ("");
+  gtk_widget_show (w);
   gtk_container_add (GTK_CONTAINER (box), w);
   gtk_widget_set_margin_end (w, 12);
   self->current_label = w;
 
   w = gtk_switch_new ();
+  gtk_widget_show (w);
   gtk_widget_set_margin_top (w, 3);
   gtk_widget_set_margin_bottom (w, 3);
   gtk_widget_set_margin_end (w, 12);
@@ -283,8 +289,6 @@ cc_sharing_networks_new_current_row (CcSharingNetworks *self)
   self->current_switch = w;
   g_object_set_data (G_OBJECT (w), "row", row);
 
-  gtk_widget_show_all (box);
-
   return row;
 }
 
@@ -295,18 +299,18 @@ cc_sharing_networks_new_no_network_row (CcSharingNetworks *self)
 
   row = gtk_list_box_row_new ();
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_widget_show (box);
   gtk_container_set_border_width (GTK_CONTAINER (box), 12);
   gtk_container_add (GTK_CONTAINER (row), box);
 
   /* Label */
   w = gtk_label_new (_("No networks selected for sharing"));
+  gtk_widget_show (w);
   gtk_widget_set_hexpand (w, TRUE);
   gtk_widget_set_halign (w, GTK_ALIGN_CENTER);
   gtk_style_context_add_class (gtk_widget_get_style_context (w), "dim-label");
   gtk_container_add (GTK_CONTAINER (box), w);
 
-  gtk_widget_show_all (box);
-
   return row;
 }
 
@@ -380,6 +384,7 @@ cc_sharing_update_networks_box (CcSharingNetworks *self)
 				       net->network_name,
 				       net->carrier_type,
 				       self);
+    gtk_widget_show (row);
     gtk_list_box_insert (GTK_LIST_BOX (self->listbox), row, -1);
   }
 
