@@ -73,10 +73,11 @@ cc_stream_row_init (CcStreamRow *self)
 }
 
 CcStreamRow *
-cc_stream_row_new (GtkSizeGroup   *size_group,
-                   GvcMixerStream *stream,
-                   guint           id,
-                   CcStreamType    stream_type)
+cc_stream_row_new (GtkSizeGroup    *size_group,
+                   GvcMixerStream  *stream,
+                   guint            id,
+                   CcStreamType     stream_type,
+                   GvcMixerControl *mixer_control)
 {
   CcStreamRow *self;
   g_autoptr(GtkIconInfo) icon_info = NULL;
@@ -110,6 +111,7 @@ cc_stream_row_new (GtkSizeGroup   *size_group,
 
   gtk_label_set_label (self->name_label, gvc_mixer_stream_get_name (stream));
   cc_volume_slider_set_stream (self->volume_slider, stream, stream_type);
+  cc_volume_slider_set_mixer_control (self->volume_slider, mixer_control);
 
   gtk_size_group_add_widget (size_group, GTK_WIDGET (self->label_box));
 
